// AuxiliaryMethods.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <cassert>
#include <vector>
#include "Common.h"
#include "OrderPlacing.h"
#include "OrderPlacingDlg.h"
#include "BracketPropertiesDlg.h"

namespace
{
ATL::CComPtr<ICQGQuote> GetValidQuote(ATL::CComPtr<ICQGQuotes>& spQuotes, eQuoteType type = qtTrade)
{
   ATL::CComPtr<ICQGQuote> spQuote;
   auto hr = spQuotes->get_Item(type, &spQuote);
   AssertCOMError(spQuotes, hr);

   VARIANT_BOOL isValid;
   hr = spQuote->get_IsValid(&isValid);
   AssertCOMError(spQuote, hr);

   if (isValid == VARIANT_TRUE)
   {
      return spQuote;
   }

   switch (type)
   {
   case qtTrade:
      return GetValidQuote(spQuotes, qtAsk);
   case qtAsk:
      return GetValidQuote(spQuotes, qtBid);
   case qtBid:
   default:
      return {};
   }
}

/// \brief Converts local machine DateTime to UTC.
/// \param i_localDateTime in, Local machine time.
COleDateTime convertToUtc(const COleDateTime& i_localDateTime)
{
   SYSTEMTIME timeLocal{};
   SYSTEMTIME timeUTC{};
   i_localDateTime.GetAsSystemTime(timeLocal);

   if (TzSpecificLocalTimeToSystemTime(nullptr, &timeLocal, &timeUTC))
   {
      return COleDateTime(timeUTC);
   }

   _ASSERT(!"Cannot convert to UTC time");
   return i_localDateTime;
}
}

void COrderPlacingDlg::InitializeCQGCEL()
{
   // create an instance of CQG API
   HRESULT hr = m_spCQGCEL.CoCreateInstance(__uuidof(CQGCEL), NULL, CLSCTX_INPROC_SERVER);
   if (FAILED(hr))
   {
      throw std::runtime_error("Unable to create CQGCEL object. "
                               "Please register it again and restart application.");
   }

   // Configure CQGCEL behavior
   ATL::CComPtr<ICQGAPIConfig> spConf;
   hr = m_spCQGCEL->get_APIConfiguration(&spConf);
   AssertCOMError(m_spCQGCEL, hr);

   hr = spConf->put_ReadyStatusCheck(rscOff);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UsedFromATLClient(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_CollectionsThrowException(VARIANT_FALSE);
   AssertCOMError(spConf, hr);

   hr = spConf->put_TimeZoneCode(tzCentral);
   AssertCOMError(spConf, hr);

   hr = spConf->put_UseOrderSide(VARIANT_TRUE);
   AssertCOMError(spConf, hr);

   // Now advise the connection, to get events
   hr = ICQGCELDispEventImpl::DispEventAdvise(m_spCQGCEL);
   ATLASSERT(SUCCEEDED(hr));

   hr = m_spCQGCEL->Startup();
   AssertCOMError(m_spCQGCEL, hr);
}

void COrderPlacingDlg::InitializeOrdersList()
{
   m_lvOrders.SetExtendedStyle(LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_HEADERDRAGDROP);
   m_lvOrders.ModifyStyle(0, LVS_SHOWSELALWAYS | LVS_SINGLESEL);

   m_lvOrders.InsertColumn(ocID, _T("Order #"), LVCFMT_LEFT, 55);
   m_lvOrders.InsertColumn(ocSide, _T("Buy/Sell"), LVCFMT_LEFT, 50);
   m_lvOrders.InsertColumn(ocSize, _T("Size"), LVCFMT_LEFT, 50);
   m_lvOrders.InsertColumn(ocState, _T("State"), LVCFMT_LEFT, 100);
   m_lvOrders.InsertColumn(ocInstrument, _T("Instrument"), LVCFMT_LEFT, 100);
   m_lvOrders.InsertColumn(ocType, _T("Type"), LVCFMT_LEFT, 58);
   m_lvOrders.InsertColumn(ocLimitPrice, _T("Limit Price"), LVCFMT_LEFT, 65);
   m_lvOrders.InsertColumn(ocStopPrice, _T("Stop Price"), LVCFMT_LEFT, 65);
   m_lvOrders.InsertColumn(ocDuration, _T("Duration"), LVCFMT_LEFT, 60);
   m_lvOrders.InsertColumn(ocGoodTill, _T("Good Till"), LVCFMT_LEFT, 150);
   m_lvOrders.InsertColumn(ocAvgFillPrice, _T("Avg Fill Price"), LVCFMT_LEFT, 75);
   m_lvOrders.InsertColumn(ocPlaceTime, _T("Place Time"), LVCFMT_LEFT, 150);
   // Hidden column
   m_lvOrders.InsertColumn(ocGWOrderID, _T("GWOrderID"), LVCFMT_LEFT, 0);
}

void COrderPlacingDlg::InitializeTypesCombo()
{
   int row = m_cmbType.AddString(_T("Market"));
   m_cmbType.SetItemData(row, otMarket);

   row = m_cmbType.AddString(_T("Limit"));
   m_cmbType.SetItemData(row, otLimit);

   row = m_cmbType.AddString(_T("Stop"));
   m_cmbType.SetItemData(row, otStop);

   row = m_cmbType.AddString(_T("StopLimit"));
   m_cmbType.SetItemData(row, otStopLimit);

   // Select first element
   m_cmbType.SetCurSel(0);
}

void COrderPlacingDlg::InitializeDurationsCombo()
{
   int row = m_cmbDuration.AddString(_T("DAY"));
   m_cmbDuration.SetItemData(row, odDay);

   row = m_cmbDuration.AddString(_T("FAK"));
   m_cmbDuration.SetItemData(row, odFAK);

   row = m_cmbDuration.AddString(_T("FOK"));
   m_cmbDuration.SetItemData(row, odFOK);
   
   row = m_cmbDuration.AddString(_T("GTC"));
   m_cmbDuration.SetItemData(row, odGoodTillCanceled);
   
   row = m_cmbDuration.AddString(_T("GTD"));
   m_cmbDuration.SetItemData(row, odGoodTillDate);
   
   row = m_cmbDuration.AddString(_T("ATC"));
   m_cmbDuration.SetItemData(row, odATC);

   row = m_cmbDuration.AddString(_T("ATO"));
   m_cmbDuration.SetItemData(row, odATO);
   
   row = m_cmbDuration.AddString(_T("GTT"));
   m_cmbDuration.SetItemData(row, odGoodTillTime);

   row = m_cmbDuration.AddString(_T("DAY GTT"));
   m_cmbDuration.SetItemData(row, odDayGoodTillTime);

   // Select first element
   m_cmbDuration.SetCurSel(0);
}

void COrderPlacingDlg::InitializeOrdersTab()
{
   m_tabOrders.InsertItem(ftWorking, _T("Working"));
   m_tabOrders.InsertItem(ftFilled, _T("Filled"));
   m_tabOrders.InsertItem(ftCancelled, _T("Cancelled"));
   m_tabOrders.InsertItem(ftExceptions, _T("Exceptions"));
   m_tabOrders.InsertItem(ftParked, _T("Parked"));
   int tab = m_tabOrders.InsertItem(ftAll, _T("All"));

   // Select "All" tab
   m_tabOrders.SetCurSel(tab);
}

void COrderPlacingDlg::InitializeBooleans()
{
   m_DataConnectionUp = false;
   m_GWConnectionUp = false;
   m_LimitPriceEnabled = false;
   m_StopPriceEnabled = false;
   m_isOrderGoodTillEnabled = false;
}

void COrderPlacingDlg::InitializeControls()
{
   ClearAllInstrumentData();

   OnDataConnectionStatusChanged(csConnectionDown);
   OnGWConnectionStatusChanged(csConnectionDown);

   InitializeOrdersTab();
   InitializeOrdersList();
   InitializeTypesCombo();
   InitializeDurationsCombo();

   m_dtpOrderGoodTill.SetFormat(m_SHORT_DATE);
}

void COrderPlacingDlg::EnableControls()
{
   m_cmbAccounts.EnableWindow(m_GWConnectionUp);

   m_edSymbolName.EnableWindow(m_DataConnectionUp);
   m_btnSubscribe.EnableWindow(m_DataConnectionUp);

   bool isInstrumentSelected = (m_spCQGCEL && GetSelectedInstrument());  

   m_lsInstruments.EnableWindow(m_DataConnectionUp);
   m_btnRemoveAll.EnableWindow(m_DataConnectionUp);
   m_btnRemove.EnableWindow(m_DataConnectionUp && isInstrumentSelected);

   bool isAccountSelected = (m_spCQGCEL && GetSelectedAccount());

   m_btnBuy.EnableWindow(m_GWConnectionUp && isAccountSelected && isInstrumentSelected);
   m_btnSell.EnableWindow(m_GWConnectionUp && isAccountSelected && isInstrumentSelected);

   m_edQuantity.EnableWindow(m_GWConnectionUp && isInstrumentSelected);
   m_spnQuantity.EnableWindow(m_GWConnectionUp && isInstrumentSelected);

   m_edLimitPrice.EnableWindow(m_GWConnectionUp && isInstrumentSelected && m_LimitPriceEnabled);
   m_spnLimitPrice.EnableWindow(m_GWConnectionUp && isInstrumentSelected && m_LimitPriceEnabled);
   
   m_edStopPrice.EnableWindow(m_GWConnectionUp && isInstrumentSelected && m_StopPriceEnabled);
   m_spnStopPrice.EnableWindow(m_GWConnectionUp && isInstrumentSelected && m_StopPriceEnabled);
   
   m_cmbType.EnableWindow(m_GWConnectionUp && isInstrumentSelected);

   m_cmbDuration.EnableWindow(m_GWConnectionUp && isInstrumentSelected);
   m_dtpOrderGoodTill.EnableWindow(m_GWConnectionUp && isInstrumentSelected && m_isOrderGoodTillEnabled);

   m_chkParked.EnableWindow(m_GWConnectionUp && isInstrumentSelected);
   m_chkBracket.EnableWindow(m_GWConnectionUp && isInstrumentSelected);

   bool isOrderSelected = (m_spSelectedOrder != NULL);

   m_btnModify.EnableWindow(m_GWConnectionUp && isOrderSelected);
   m_btnActivate.EnableWindow(m_GWConnectionUp && isOrderSelected);
   m_btnActivateAll.EnableWindow(m_GWConnectionUp && isAccountSelected);
   m_btnCancel.EnableWindow(m_GWConnectionUp && isOrderSelected);
   m_btnCancelAll.EnableWindow(m_GWConnectionUp && isAccountSelected);
}

void COrderPlacingDlg::ClearAllInstrumentData()
{
   ClearDescription();
   ClearQuotes();

   m_edStopPrice.SetWindowText(m_N_A);
   m_edLimitPrice.SetWindowText(m_N_A);

   SetQuantity(m_edQuantity, m_MIN_QUANTITY);
}

void COrderPlacingDlg::ClearDescription()
{
   m_lblInstrDescription.SetWindowText(m_N_A);
}

void COrderPlacingDlg::ClearQuotes()
{
   // Set all quotes texts to N/A
   m_lblAskPrice.SetWindowText(m_N_A);
   m_lblAskVolume.SetWindowText(m_N_A);
   m_lblBidPrice.SetWindowText(m_N_A);
   m_lblBidVolume.SetWindowText(m_N_A);
   m_lblTradePrice.SetWindowText(m_N_A);
   m_lblTradeVolume.SetWindowText(m_N_A);
}

void COrderPlacingDlg::ClearOrders()
{
   // Clear orders
   m_lvOrders.DeleteAllItems();

   m_spSelectedOrder = NULL;
}

void COrderPlacingDlg::ReloadAccounts()
{
   // Clear all previous data
   m_cmbAccounts.ResetContent();
   ClearOrders();

   // Get accounts from the CQGCEL
   ATL::CComPtr<ICQGAccounts> spAccounts;
   HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
   AssertCOMError(m_spCQGCEL, hr);

   long count;
   hr = spAccounts->get_Count(&count);
   AssertCOMError(spAccounts, hr);

   if (count == 0)
   {
      // No accounts
      EnableControls();
      return;
   }

   // Iterate over accounts collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spAccounts->get__NewEnum(&spEnumVariant);
   AssertCOMError(spAccounts, hr);

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      ATLASSERT(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGAccount> spAccount;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGAccount), (void** )&spAccount);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current account to the list
      InsertAccount(spAccount);
   }

   // Select the first account in the list
   m_cmbAccounts.SetCurSel(0);

   EnableControls();
}

void COrderPlacingDlg::InsertAccount(
                                 const ATL::CComPtr<ICQGAccount>& spAccount)
{
   // Create string for specified account to add to the combo
   ATL::CComBSTR accName;
   HRESULT hr = spAccount->get_GWAccountName(&accName);
   AssertCOMError(spAccount, hr);

   ATL::CComBSTR FcmAccID;
   hr = spAccount->get_FcmAccountID(&FcmAccID);
   AssertCOMError(spAccount, hr);

   CString accString = ATL::CW2T(accName);
   accString = accString + _T(" (") + ATL::CW2T(FcmAccID) + _T(")");

   // Get account ID to map with item
   long accID;
   hr = spAccount->get_GWAccountID(&accID);
   AssertCOMError(spAccount, hr);

   // Add new account name to the combo and map ID
   int row = m_cmbAccounts.AddString(accString);
   m_cmbAccounts.SetItemData(row, accID);
}

void COrderPlacingDlg::ShowDescription(
                                 const ATL::CComPtr<ICQGInstrument>& spInstrument)
{
   ATL::CComBSTR instrDescr;
   HRESULT hr = spInstrument->get_Description(&instrDescr);
   AssertCOMError(spInstrument, hr);

   m_lblInstrDescription.SetWindowText(CW2T(instrDescr));
}

double COrderPlacingDlg::GetValidPrice(
                                 const ATL::CComPtr<ICQGInstrument>& spInstrument)
{
   ATL::CComPtr<ICQGQuotes> spQuotes;
   HRESULT hr = spInstrument->get_Quotes(&spQuotes);
   AssertCOMError(spInstrument, hr);
 
   ATL::CComPtr<ICQGQuote> spQuote = GetValidQuote(spQuotes);
   
   double price = m_INVALID_DOUBLE;

   if (spQuote)
   {
      hr = spQuote->get_Price(&price);
      AssertCOMError(spQuote, hr);
   }
   
   return price;
}

void COrderPlacingDlg::ShowValidPrice(
                                 const ATL::CComPtr<ICQGInstrument>& spInstrument)
{   
   m_edLimitPrice.SetWindowText(m_N_A);
   m_edStopPrice.SetWindowText(m_N_A);
   
      // Get order type
   eOrderType orderType = static_cast<eOrderType>
                                    (m_cmbType.GetItemData(m_cmbType.GetCurSel()));

   m_LimitPriceEnabled = (orderType == otLimit || orderType == otStopLimit);
   m_StopPriceEnabled = (orderType == otStop || orderType == otStopLimit);
   
   if (m_LimitPriceEnabled || m_StopPriceEnabled)
   {
      VARIANT_BOOL isValid;
      double price = GetValidPrice(spInstrument);
      
      HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(price), &isValid);
      AssertCOMError(m_spCQGCEL, hr);
      
      if (isValid)
      {
         ATL::CComBSTR displayPrice;
         hr = spInstrument->ToDisplayPrice(price, &displayPrice);
         AssertCOMError(spInstrument, hr);
         
         if (m_LimitPriceEnabled)
         {
            m_edLimitPrice.SetWindowText(ATL::CW2T(displayPrice));
         }
            
         if (m_StopPriceEnabled)
         {
            m_edStopPrice.SetWindowText(ATL::CW2T(displayPrice));
         }
      }
   }
   
   EnableControls();
}

void COrderPlacingDlg::ShowQuotes(
                                 const ATL::CComPtr<ICQGInstrument>& spInstrument)
{
   ATL::CComPtr<ICQGQuotes> spQuotes;
   HRESULT hr = spInstrument->get_Quotes(&spQuotes);
   AssertCOMError(spInstrument, hr);

   ShowQuote(spQuotes, qtAsk, m_lblAskPrice, m_lblAskVolume);
   ShowQuote(spQuotes, qtTrade, m_lblTradePrice, m_lblTradeVolume);
   ShowQuote(spQuotes, qtBid, m_lblBidPrice, m_lblBidVolume);
}

void COrderPlacingDlg::ShowQuote(
                                 const ATL::CComPtr<ICQGQuotes>& spQuotes,
                                 eQuoteType quoteType,
                                 CStatic& priceLabel,
                                 CStatic& volumeLabel)
{
   // Update quote
   ATL::CComPtr<ICQGQuote> spQuote;
   HRESULT hr = spQuotes->get_Item(quoteType, &spQuote);
   if (SUCCEEDED(hr))
   {
      CString displayPrice;
      CString displayVolume;
      GetQuoteDisplayValues(spQuote, displayPrice, displayVolume); 
      priceLabel.SetWindowText(displayPrice);
      volumeLabel.SetWindowText(displayVolume);

      spQuote.Release();
   }
}

void COrderPlacingDlg::GetQuoteDisplayValues(
                                 const ATL::CComPtr<ICQGQuote>& spQuote,
                                 CString& displayPrice,
                                 CString& displayVolume)
{
   displayPrice = m_N_A;
   displayVolume = m_N_A;

   if (spQuote != NULL)
   {
      VARIANT_BOOL isValid;
      HRESULT hr = spQuote->get_IsValid(&isValid);
      AssertCOMError(spQuote, hr);
      if (isValid)
      {
         double price;
         hr = spQuote->get_Price(&price);
         AssertCOMError(spQuote, hr);

         long volume;
         hr = spQuote->get_Volume(&volume);
         AssertCOMError(spQuote, hr);

         ATL::CComPtr<ICQGInstrument> spInstrument;
         hr = spQuote->get_Instrument(&spInstrument);
         AssertCOMError(spQuote, hr);

         // Get the display price from price
         ATL::CComBSTR dispPrice;
         hr = spInstrument->ToDisplayPrice(price, &dispPrice);
         AssertCOMError(spInstrument, hr);

         // Convert volume to string
         CString dispVolume;
         dispVolume.Format(_T("%d"), volume);

         displayPrice = ATL::CW2T(dispPrice);
         displayVolume = dispVolume;
      }
   }
}

void COrderPlacingDlg::ShowFilteredOrders()
{
   // Clear old data
   ClearOrders();

   // Get account and it's all orders
   ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();
   
   if (spAccount == NULL)
   {
      // No account selected
      return;
   }

   ATL::CComPtr<ICQGOrders> spOrders;
   HRESULT hr = spAccount->get_InternalOrders(&spOrders);
   AssertCOMError(spOrders, hr);

   // Get order status, to filter by
   int status = FilterTypeToOrderStatus(static_cast<eFilterType>(m_tabOrders.GetCurSel()));

   if (status == m_INVALID_INTEGER)
   {
      // No filter, show all orders
      ShowOrders(spOrders);
   }
   else
   {
      // Get filtered orders by selected status
      ATL::CComPtr<ICQGOrders> spOrdersBy;
      hr = spOrders->SelectByOrderStatus(static_cast<eOrderStatus>(status), &spOrdersBy);
      AssertCOMError(spOrders, hr);

      // Show filtered orders
      ShowOrders(spOrdersBy);
   }
}

void COrderPlacingDlg::UnselectAllOrders()
{
   for (int i = 0; i < m_lvOrders.GetItemCount(); ++i)
   {
      m_lvOrders.SetItemState(i, 0, LVIS_SELECTED);
   }
}

void COrderPlacingDlg::ShowOrders(
                                 const ATL::CComPtr<ICQGOrders>& spOrders)
{
   long count;
   HRESULT hr = spOrders->get_Count(&count);
   AssertCOMError(spOrders, hr);

   if (count == 0)
   {
      // No orders
      return;
   }

   // Iterate over orders collection and add them to the list
   ATL::CComPtr<IEnumVARIANT> spEnumVariant;
   hr = spOrders->get__NewEnum(&spEnumVariant);
   AssertCOMError(spOrders, hr);

   std::vector<ATL::CComVariant> pItems(count);

   ULONG celtFetched;
   hr = spEnumVariant->Next(count, &pItems[0], &celtFetched);
   AssertCOMError(spEnumVariant, hr);

   for (std::vector<ATL::CComVariant>::iterator it = pItems.begin(); it != pItems.end(); ++it)
   {
      assert(it->vt == VT_DISPATCH);

      ATL::CComPtr<ICQGOrder> spOrder;
      hr = it->pdispVal->QueryInterface(__uuidof(ICQGOrder), (void** )&spOrder);
      AssertCOMError<IDispatch>(it->pdispVal, hr);

      // Add current order to the list
      ShowOrder(spOrder, ctAdded);
   }
}

int COrderPlacingDlg::GetOrderRow(
                                 const ATL::CComPtr<ICQGOrder>& spOrder)
{
   ATL::CComBSTR orderID;
   HRESULT hr = spOrder->get_OriginalOrderID(&orderID);
   AssertCOMError(spOrder, hr);

   // Find row for specified order
   LVFINDINFO findInfo;
   findInfo.flags = LVFI_STRING;
   findInfo.psz = ATL::CW2T(orderID);

   return m_lvOrders.FindItem(&findInfo);
}

void COrderPlacingDlg::ShowOrder(
                                 const ATL::CComPtr<ICQGOrder>& spOrder,
                                 eChangeType change)

{
   // Get specified order's row
   int row = GetOrderRow(spOrder);

   // If change type is removed, remove order from list
   if (change == ctRemoved && row > -1)
   {
      m_lvOrders.DeleteItem(row);
      m_spSelectedOrder = NULL;

      return;
   }

   // Add new one if not found
   if (row == -1)
   {
      row = m_lvOrders.InsertItem(m_lvOrders.GetItemCount(), _T(""));
   }

   ATL::CComBSTR orderID;
   HRESULT hr = spOrder->get_OriginalOrderID(&orderID);
   AssertCOMError(spOrder, hr);

   eOrderSide side;
   hr = spOrder->get_Side(&side);
   AssertCOMError(spOrder, hr);

   ATL::CComVariant size;
   hr = spOrder->get_QuantityFractional(&size.decVal);
   AssertCOMError(spOrder, hr);

   eOrderStatus status;
   hr = spOrder->get_GWStatus(&status);
   AssertCOMError(spOrder, hr);

   ATL::CComBSTR instrumentName;
   hr = spOrder->get_InstrumentName(&instrumentName);
   AssertCOMError(spOrder, hr);

   eOrderType type;
   hr = spOrder->get_Type(&type);
   AssertCOMError(spOrder, hr);

   double limitPrice;
   hr = spOrder->get_LimitPrice(&limitPrice);
   AssertCOMError(spOrder, hr);

   double stopPrice;
   hr = spOrder->get_StopPrice(&stopPrice);
   AssertCOMError(spOrder, hr);

   eOrderDuration duration;
   hr = spOrder->get_DurationType(&duration);
   AssertCOMError(spOrder, hr);

   DATE placeTime;
   hr = spOrder->get_PlaceTime(&placeTime);
   AssertCOMError(spOrder, hr);

   // Get order properties to get average fill price
   ATL::CComPtr<ICQGOrderProperties> spProperties;
   hr = spOrder->get_Properties(&spProperties);
   AssertCOMError(spOrder, hr);

   ATL::CComPtr<ICQGOrderProperty> spProperty;
   hr = spProperties->get_Item(opAverageFillPrice, &spProperty);
   AssertCOMError(spProperties, hr);

   double avgFillPrice{};
   if (SUCCEEDED(hr) && hr != S_FALSE)
   {
      ATL::CComVariant avgFillPriceVar;
      hr = spProperty->get_Value(&avgFillPriceVar);
      AssertCOMError(spProperty, hr);

      avgFillPrice = avgFillPriceVar.dblVal;
   }

   spProperty = nullptr;
   CString goodTill;
   if (duration == odGoodTillDate || duration == odGoodTillTime || duration == odDayGoodTillTime)
   {
      hr = spProperties->get_Item(duration == odGoodTillDate ? opGTDTime : opGTTTime, &spProperty);
      AssertCOMError(spProperties, hr);

      if (SUCCEEDED(hr) && hr != S_FALSE)
      {
         ATL::CComVariant goodTillVar;
         hr = spProperty->get_Value(&goodTillVar);
         AssertCOMError(spProperty, hr);

         goodTill = COleDateTime(goodTillVar.date).Format(duration == odGoodTillDate ? VAR_DATEVALUEONLY : 0);
      }
   }

   m_lvOrders.SetItemText(row, ocID, ATL::CW2T(orderID));
   m_lvOrders.SetItemText(row, ocSide, OrderSideToString(side));
   m_lvOrders.SetItemText(row, ocSize, GetValueAsString(size, VT_DECIMAL));
   m_lvOrders.SetItemText(row, ocState, OrderStatusToString(status));
   m_lvOrders.SetItemText(row, ocInstrument, ATL::CW2T(instrumentName));
   m_lvOrders.SetItemText(row, ocType, OrderTypeToString(type));
   m_lvOrders.SetItemText(row, ocLimitPrice, GetDisplayPrice(spOrder, limitPrice));
   m_lvOrders.SetItemText(row, ocStopPrice, GetDisplayPrice(spOrder, stopPrice));
   m_lvOrders.SetItemText(row, ocDuration, OrderDurationToString(duration));
   m_lvOrders.SetItemText(row, ocGoodTill, goodTill);
   m_lvOrders.SetItemText(row, ocAvgFillPrice, GetDisplayPrice(spOrder, avgFillPrice));
   m_lvOrders.SetItemText(row, ocPlaceTime, COleDateTime(placeTime).Format());
  
   // Map order GW ID with corresponding item
   ATL::CComBSTR gwOrderID;
   hr = spOrder->get_GWOrderID(&gwOrderID);
   AssertCOMError(spOrder, hr);

   m_lvOrders.SetItemText(row, ocGWOrderID, ATL::CW2T(gwOrderID));
}

CString COrderPlacingDlg::GetDisplayPrice(
                                 const ATL::CComPtr<ICQGOrder> spOrder,
                                 double price) const
{
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(ATL::CComVariant(price), &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   // Get instrument to get price display format
   ATL::CComPtr<ICQGInstrument> spInstrument;
   hr = spOrder->get_Instrument(&spInstrument);
   AssertCOMError(spOrder, hr);

   CString displayPrice;
   if (isValid && spInstrument != NULL)
   {
      // Get display price
      ATL::CComBSTR displayPriceBSTR;
      hr = spInstrument->ToDisplayPrice(price, &displayPriceBSTR);
      AssertCOMError(spInstrument, hr);

      displayPrice = ATL::CW2T(displayPriceBSTR);
   }
   else
   {
      displayPrice = m_N_A;
   }

   return displayPrice;
}

CString COrderPlacingDlg::GetValueAsString(
                                 ATL::CComVariant value,
                                 VARENUM valueType/*= VT_EMPTY*/) const
{
   if (valueType != VT_EMPTY)
   {
      value.vt = static_cast<VARTYPE>(valueType);
   }

   // Check validness
   VARIANT_BOOL isValid;
   HRESULT hr = m_spCQGCEL->IsValid(value, &isValid);
   AssertCOMError(m_spCQGCEL, hr);

   if (isValid)
   {
      return VariantToString(value);
   }
   return m_N_A;
}

ATL::CComPtr<ICQGAccount> COrderPlacingDlg::GetSelectedAccount() const
{
   try
   {
      ATL::CComPtr<ICQGAccounts> spAccounts;
      HRESULT hr = m_spCQGCEL->get_Accounts(&spAccounts);
      AssertCOMError(m_spCQGCEL, hr);

      long accID = static_cast<long>(m_cmbAccounts.GetItemData(m_cmbAccounts.GetCurSel()));

      ATL::CComPtr<ICQGAccount> spAccount;
      hr = spAccounts->get_Item(accID, &spAccount);
      AssertCOMError(spAccounts, hr);

      return spAccount;
   }
   catch (...)
   {
   }

   return NULL;
}

ATL::CComPtr<ICQGInstrument> COrderPlacingDlg::GetSelectedInstrument() const
{
   try
   {
      // Get selected instrument's row
      int row = m_lsInstruments.GetCurSel();
      if (row > -1)
      {
         // Get text from the selected row
         CString curText;
         m_lsInstruments.GetText(row, curText);

         ATL::CComBSTR fullName = ATL::CT2W(curText);

         // Get instruments collection
         ATL::CComPtr<ICQGInstruments> spInstruments;
         HRESULT hr = m_spCQGCEL->get_Instruments(&spInstruments);
         AssertCOMError(m_spCQGCEL, hr);

         // Get instrument from collection by it's full name
         ATL::CComVariant idx(fullName);
         ATL::CComPtr<ICQGInstrument> spInstrument;
         hr = spInstruments->get_Item(idx, &spInstrument);
         AssertCOMError(m_spCQGCEL, hr);

         return spInstrument;
      }
   }
   catch (...)
   {
   }

   return NULL;
}

ATL::CComPtr<ICQGOrder> COrderPlacingDlg::CreateOrder(
   const ATL::CComPtr<ICQGInstrument>& instrument,
   const ATL::CComPtr<ICQGAccount>& account,
   eOrderType orderType,
   eOrderSide orderSide,
   double stopPrice,
   double limitPrice)
{
   // Create order constructor and fill needed properties
   ATL::CComPtr<ICQGOrderConstructor> constructor;
   HRESULT hr = m_spCQGCEL->CreateOrderConstructor(orderType, orderSide, &constructor);
   AssertCOMError(m_spCQGCEL, hr);

   ATL::CComPtr<ICQGOrderProperties> properties;
   hr = constructor->get_Properties(&properties);
   AssertCOMError(constructor, hr);

   _bstr_t instrumentName;
   hr = instrument->get_DetailedName(instrumentName.GetAddress());
   AssertCOMError(m_spCQGCEL, hr);

   SetPropertyValue(properties, opInstrumentNameDetailed, _variant_t(instrumentName));
   long GWAccountID;
   hr = account->get_GWAccountID(&GWAccountID);
   AssertCOMError(m_spCQGCEL, hr);

   SetPropertyValue(properties, opAccountID, GWAccountID);
   SetPropertyValue(properties, opUEName, _variant_t(_bstr_t()));

   // Set quantity
   SetPropertyValue(properties, opQuantityFractional, _variant_t(GetQuantity(m_edQuantity)));

   if (orderType == otStop || orderType == otStopLimit)
   {
      SetPropertyValue(properties, opStopPrice, stopPrice);
   }
   if (orderType == otLimit || orderType == otStopLimit)
   {
      SetPropertyValue(properties, opLimitPrice, limitPrice);
   }

   ATL::CComPtr<ICQGOrder> order;
   hr = constructor->CreateOrder(&order);
   AssertCOMError(constructor, hr);

   return order;
}

void COrderPlacingDlg::PlaceOrder(
                                 eOrderSide orderSide)
{
   ATL::CComPtr<ICQGAccount> spAccount = GetSelectedAccount();

   ATL::CComPtr<ICQGInstrument> spInstrument = GetSelectedInstrument();

   // Get order type
   eOrderType orderType = static_cast<eOrderType>
                                     (m_cmbType.GetItemData(m_cmbType.GetCurSel()));

   // Get limit price and stop price 
   double limitPrice = 0;
   double stopPrice = 0;

   switch(orderType)
   {
      case otLimit:
         limitPrice = GetPrice(m_edLimitPrice, spInstrument);
         break;
      case otStop:
         stopPrice = GetPrice(m_edStopPrice, spInstrument);
         break;
      case otStopLimit:
         limitPrice = GetPrice(m_edLimitPrice, spInstrument);
         stopPrice = GetPrice(m_edStopPrice, spInstrument);
   }

   if (limitPrice == m_INVALID_DOUBLE || stopPrice == m_INVALID_DOUBLE)
   {
      // Limit price or/and stop price is incorrect
      AfxMessageBox(_T("Incorrect price for '") + OrderTypeToString(orderType) + _T("' order"));
      return;
   }

   ATL::CComPtr<ICQGOrder> spOrder = CreateOrder(spInstrument, spAccount, orderType, orderSide, stopPrice, limitPrice);

   // Put duration type of the order
   eOrderDuration orderDuration = static_cast<eOrderDuration>
                                    (m_cmbDuration.GetItemData(m_cmbDuration.GetCurSel()));
   HRESULT hr = spOrder->put_DurationType(orderDuration);
   AssertCOMError(m_spCQGCEL, hr);

   // Get order properties to change some values
   ATL::CComPtr<ICQGOrderProperties> spProperties;
   hr = spOrder->get_Properties(&spProperties);
   AssertCOMError(spOrder, hr);

   if (orderDuration == odGoodTillDate || orderDuration == odGoodTillTime || orderDuration == odDayGoodTillTime)
   {
      // You should always pass UTC time to API calls
      COleDateTime oleDateTime;
      m_dtpOrderGoodTill.GetTime(oleDateTime);
      const auto utcTime = convertToUtc(oleDateTime);
      SetPropertyValue(spProperties, orderDuration == odGoodTillDate ? opGTDTime : opGTTTime,
                       ATL::CComVariant(utcTime, VT_DATE));
   }

   // Set parked property
   SetPropertyValue(spProperties, opParked, ATL::CComVariant(m_chkParked.GetCheck() == BST_CHECKED));

   if (m_chkBracket.GetCheck() == BST_CHECKED)
   {
      // Open bracket properties dialog
      CBracketPropertiesDlg dlg(m_spCQGCEL, spOrder, orderSide);
      dlg.DoModal();
   }
   else
   {
      // Place order
      hr = spOrder->Place();
      AssertCOMError(spOrder, hr);
   }
}

int COrderPlacingDlg::FilterTypeToOrderStatus(
                                 eFilterType filterType)
{
   int orderStatus = m_INVALID_INTEGER;

   switch(filterType)
   {
      case ftWorking:
         orderStatus = osInOrderBook;
         break;
      case ftFilled:
         orderStatus = osFilled;
         break;
      case ftCancelled:
         orderStatus = osCanceled;
         break;
      case ftExceptions:
         orderStatus = osRejectGW;
         break;
      case ftParked:
         orderStatus = osParked;
         break;
      case ftAll:
         orderStatus = m_INVALID_INTEGER;
   }

   return orderStatus;
}

CString COrderPlacingDlg::OrderDurationToString(
                                 eOrderDuration duration)
{
   CString durationString;

   switch (duration)
   {
      case odATC:
         durationString = _T("ATC");
         break;
      case odATO:
         durationString = _T("ATO");
         break;
      case odDay:
         durationString = _T("DAY");
         break;
      case odFAK:
         durationString = _T("FAK");
         break;
      case odFOK:
         durationString = _T("FOK");
         break;
      case odGoodTillCanceled:
         durationString = _T("GTC");
         break;
      case odGoodTillDate:
         durationString = _T("GTD");
         break;
      case odGoodTillTime:
         durationString = _T("GTT");
         break;
      case odDayGoodTillTime:
         durationString = _T("DAY GTT");
         break;
      case odUndefined:
         durationString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order duration");
   }

   return durationString;
}

CString COrderPlacingDlg::OrderSideToString(
                                 eOrderSide side)
{
   CString sideString;

   switch (side)
   {
      case osdBuy:
         sideString = _T("Buy");
         break;
      case osdSell:
         sideString = _T("Sell");
         break;
      case osdUndefined:
         sideString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order side");
   }

   return sideString;
}

CString COrderPlacingDlg::OrderTypeToString(
                                 eOrderType type)
{
   CString typeString;

   switch (type)
   {
      case otLimit:
         typeString = _T("Limit");
         break;
      case otMarket:
         typeString = _T("Market");
         break;
      case otStop:
         typeString = _T("Stop");
         break;
      case otStopLimit:
         typeString = _T("Stop Limit");
         break;
      case otUndefined:
         typeString = _T("Undefined");
         break;
      default:
         assert(!"Unknown order type");
   }

   return typeString;
}

CString COrderPlacingDlg::OrderStatusToString(
                                 eOrderStatus status)
{
   CString statusString;

   switch (status)
   {
      case osActiveAt:
         statusString = _T("Active At");
         break;
      case osBusted:
         statusString = _T("Busted");
         break;
      case osCanceled:
         statusString = _T("Canceled");
         break;
      case osContingent:
         statusString = _T("Contingent");
         break;
      case osDisconnected:
         statusString = _T("Disconnected");
         break;
      case osExpired:
         statusString = _T("Expired");
         break;
      case osFilled:
         statusString = _T("Filled");
         break;
      case osInCancel:
         statusString = _T("In Cancel");
         break;
      case osInClient:
         statusString = _T("In Client");
         break;
      case osInModify:
         statusString = _T("In Modify");
         break;
      case osInOrderBook:
         statusString = _T("In Order Book");
         break;
      case osInTransit:
         statusString = _T("In Transit");
         break;
      case osInTransitTimeout:
         statusString = _T("In Transit Timeout");
         break;
      case osNotSent:
         statusString = _T("Not Sent");
         break;
      case osParked:
         statusString = _T("Parked");
         break;
      case osRejectFCM:
         statusString = _T("Reject FCM");
         break;
      case osRejectGW:
         statusString = _T("Reject GW");
         break;
      default:
         assert(!"Unknown order status");
   }

   return statusString;
}

CString COrderPlacingDlg::ConnectionStatusToString(
                                 eConnectionStatus status)
{
   CString  statusString;

   switch (status)
   {
   case csConnectionDelayed:
      statusString = _T("delayed");
      break;
   case csConnectionDown:
      statusString = _T("down");
      break;
   case csConnectionUp:
      statusString = _T("up");
      break;
   case csConnectionTrouble:
      statusString = _T("trouble");
      break;
   case csConnectionNotLoggedOn:
      statusString = _T("not logged on");
      break;
   default:
      assert(!"Unknown connection status");
   }

   return statusString;
}