// BracketPropertiesDlg.cpp : implementation file
//

#include "stdafx.h"
#include <stdexcept>
#include <string>
#include <map>
#include <sstream>
#include "Common.h"
#include "OrderPlacing.h"
#include "OrderPlacingDlg.h"
#include "BracketPropertiesDlg.h"


namespace
{
   enum class eOffsetType
   {
      oftTicks,
      oftPrice
   };
}


// CBracketPropertiesDlg dialog

IMPLEMENT_DYNAMIC(CBracketPropertiesDlg, CDialog)

CBracketPropertiesDlg::CBracketPropertiesDlg(ATL::CComPtr<ICQGCEL> cqgcel,
   ATL::CComPtr<ICQGOrder> order,
   eOrderSide orderSide,
   CWnd* pParent /*=nullptr*/)
   : CDialog(IDD_BRACKET_PROPERTIES_DIALOG, pParent),
   m_spCEL(cqgcel),
   m_spOrder(order),
   m_mainOrderSide(orderSide)
{
}

void CBracketPropertiesDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_LBL_BRKT_INFOLINE, m_lblInfoLine);
    DDX_Control(pDX, IDC_BTN_BRKT_PLACE, m_btnPlace);
    DDX_Control(pDX, IDC_BTN_BRKT_CANCEL, m_btnCancel);
    DDX_Control(pDX, IDC_LBL_BRKT_HEDGETYPE, m_lblHedge);
    DDX_Control(pDX, IDC_CMB_BRKT_SPECULATIONTYPE, m_cmbSpeculationType);
    DDX_Control(pDX, IDC_CHK_BRKT_INCLUDE_TARGET, m_chkIncludeTarget);
    DDX_Control(pDX, IDC_ED_BRKT_TARGET_TICKPRICE, m_edTargetLimitTickPrice);
    DDX_Control(pDX, IDC_SPN_BRKT_TARGET_TICKPRICE, m_spnTargetTickPrice);
    DDX_Control(pDX, IDC_CMB_BRKT_TARGET_OFFSETTYPE, m_cmbTargetOffsetType);
    DDX_Control(pDX, IDC_CHK_BRKT_INCLUDE_STOPLOSS, m_chkIncludeStopLoss);
    DDX_Control(pDX, IDC_ED_BRKT_STOPLOSS_STOP_TICKPRICE, m_edStopLossStopTickPrice);
    DDX_Control(pDX, IDC_SPN_BRKT_STOPLOSS_STOP_TICKPRICE, m_spnStopLossStopTickPrice);
    DDX_Control(pDX, IDC_CMB_BRKT_STOPLOSS_OFFSETTYPE, m_cmbStopLossOffsetType);
    DDX_Control(pDX, IDC_CMB_BRKT_STOPLOSS_ORDERTYPE, m_cmbStopLossOrderType);
    DDX_Control(pDX, IDC_ED_BRKT_STOPLOSS_LIMIT_TICKPRICE, m_edStopLossLimitTickPrice);
    DDX_Control(pDX, IDC_SPN_BRKT_STOPLOSS_LIMIT_TICKPRICE, m_spnStopLossLimitTickPrice);
    DDX_Control(pDX, IDC_LBL_BRKT_STOPLOSS_OFFSETTYPE, m_lblStopLossOffsetType);
}

BEGIN_MESSAGE_MAP(CBracketPropertiesDlg, CDialog)
   ON_CBN_SELCHANGE(IDC_CMB_BRKT_STOPLOSS_ORDERTYPE, &CBracketPropertiesDlg::OnCbnSelchangeCmbBrktStoplossOrdertype)
   ON_BN_CLICKED(IDC_CHK_BRKT_INCLUDE_STOPLOSS, &CBracketPropertiesDlg::OnBnClickedChkBrktIncludeStoploss)
   ON_BN_CLICKED(IDC_CHK_BRKT_INCLUDE_TARGET, &CBracketPropertiesDlg::OnBnClickedChkBrktIncludeTarget)
   ON_CBN_SELCHANGE(IDC_CMB_BRKT_STOPLOSS_ORDERTYPE, &CBracketPropertiesDlg::OnCbnSelchangeCmbBrktStoplossOrdertype)
   ON_CBN_SELCHANGE(IDC_CMB_BRKT_STOPLOSS_OFFSETTYPE, &CBracketPropertiesDlg::OnCbnSelchangeCmbBrktStoplossOffsettype)
   ON_BN_CLICKED(IDC_BTN_BRKT_CANCEL, &CBracketPropertiesDlg::OnBnClickedBtnBrktCancel)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_BRKT_TARGET_TICKPRICE, &CBracketPropertiesDlg::OnDeltaposSpnBrktTargetTickprice)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_BRKT_STOPLOSS_STOP_TICKPRICE, &CBracketPropertiesDlg::OnDeltaposSpnBrktStoplossStopTickprice)
   ON_NOTIFY(UDN_DELTAPOS, IDC_SPN_BRKT_STOPLOSS_LIMIT_TICKPRICE, &CBracketPropertiesDlg::OnDeltaposSpnBrktStoplossLimitTickprice)
   ON_CBN_SELCHANGE(IDC_CMB_BRKT_TARGET_OFFSETTYPE, &CBracketPropertiesDlg::OnCbnSelchangeCmbBrktTargetOffsettype)
   ON_BN_CLICKED(IDC_BTN_BRKT_PLACE, &CBracketPropertiesDlg::OnBnClickedBtnBrktPlace)
END_MESSAGE_MAP()

BOOL CBracketPropertiesDlg::OnInitDialog()
{
   try
   {
      CDialog::OnInitDialog();

      intializeControls();
      initializeSpeculationTypes();
      initializeStopLossOrderTypes();
      initializeTargetOffsetTypes();
      initializeStopLossOffsetType();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
      EndDialog(IDABORT);
   }

   return TRUE;
}

#pragma region Initializers

double CBracketPropertiesDlg::getPrice()
{
   eOrderType orderType;
   if (FAILED(m_spOrder->get_Type(&orderType)))
   {
      throw std::runtime_error("Couldn't order type.");
   }
   
   double price{};

   switch (orderType)
   {
   case otStop:
      if (SUCCEEDED(m_spOrder->get_StopPrice(&price))) return price;
      break;
   case otLimit:
   case otStopLimit:
      if (SUCCEEDED(m_spOrder->get_LimitPrice(&price))) return price;
      break;
   case otMarket:
   {
      ATL::CComPtr<ICQGInstrument> instrument;
      if (FAILED(m_spOrder->get_Instrument(&instrument)))
      {
         throw std::runtime_error("Couldn't get instrument.");
      }

      HRESULT hr = E_FAIL;
      ATL::CComPtr<ICQGQuote> quote;
      if (m_mainOrderSide == osdBuy)
      {
         hr = instrument->get_Bid(&quote);
      }
      else
      {
         hr = instrument->get_Ask(&quote);
      }

      if (FAILED(hr))
      {
         throw std::runtime_error("Couldn't get quote.");
      }

      hr = quote->get_Price(&price);
      AssertCOMError(quote, hr);
      if (price == COrderPlacingDlg::m_INVALID_DOUBLE)
      {
         throw std::runtime_error("Market price is unavailable.");
      }

      return price;
   }
   }

   throw std::runtime_error("Couldn't get price.");
}

std::string CBracketPropertiesDlg::createInfoLine()
{
   DECIMAL quantity;
   if (FAILED(m_spOrder->get_QuantityFractional(&quantity)))
   {
      throw std::runtime_error("Couldn't get quantity.");
   }

   _bstr_t instrumentName;
   if (FAILED(m_spOrder->get_InstrumentName(instrumentName.GetAddress())))
   {
      throw std::runtime_error("Couldn't get instrument name.");
   }

   eOrderType orderType;
   if (FAILED(m_spOrder->get_Type(&orderType)))
   {
      throw std::runtime_error("Couldn't get order type.");
   }

   std::map<eOrderType, std::string> orderTypes =
   {
      { otMarket,    "Market"},
      { otStop,      "Stop" },
      { otLimit,     "Limit" },
      { otStopLimit, "StopLimit"}
   };

   std::stringstream infoLine;
   infoLine << (m_mainOrderSide == osdBuy ? "BUY " : "SELL ")
      << _bstr_t(VariantToString(_variant_t(quantity))) << " "
      << instrumentName << " @ "
      << std::to_string(getPrice()) << " "
      << "BRKT "
      << orderTypes[orderType];

   return infoLine.str();
}

void CBracketPropertiesDlg::intializeControls()
{
   m_chkIncludeTarget.SetCheck(BST_CHECKED);
   m_chkIncludeStopLoss.SetCheck(BST_CHECKED);
   m_lblStopLossOffsetType.SetWindowText(_T("Ticks"));
   m_edTargetLimitTickPrice.SetWindowText(_T("1"));
   m_edStopLossStopTickPrice.SetWindowText(_T("1"));
   m_edStopLossLimitTickPrice.SetWindowText(_T("1"));
   m_lblInfoLine.SetWindowText(_bstr_t(createInfoLine().c_str()));

   _bstr_t orderSide = "Place ";
   orderSide += m_mainOrderSide == osdBuy ? "BUY " : "SELL ";
   orderSide += "Order";
   m_btnPlace.SetWindowText(orderSide);
 }

void CBracketPropertiesDlg::initializeSpeculationTypes()
{
   int row = m_cmbSpeculationType.AddString(_T("Undefined"));
   m_cmbSpeculationType.SetItemData(row, sptUndefined);

   row = m_cmbSpeculationType.AddString(_T("Speculation"));
   m_cmbSpeculationType.SetItemData(row, sptSpeculation);

   row = m_cmbSpeculationType.AddString(_T("Hedge"));
   m_cmbSpeculationType.SetItemData(row, sptHedge);

   row = m_cmbSpeculationType.AddString(_T("Arbitrage"));
   m_cmbSpeculationType.SetItemData(row, sptArbitrage);

   // Select first element
   m_cmbSpeculationType.SetCurSel(m_cmbSpeculationType.FindString(0, _T("Undefined")));
}

void CBracketPropertiesDlg::initializeStopLossOrderTypes()
{
   int row = m_cmbStopLossOrderType.AddString(_T("Stop"));
   m_cmbStopLossOrderType.SetItemData(row, otStop);

   row = m_cmbStopLossOrderType.AddString(_T("StopLimit"));
   m_cmbStopLossOrderType.SetItemData(row, otStopLimit);

   // Select first element
   m_cmbStopLossOrderType.SetCurSel(m_cmbStopLossOrderType.FindString(0, _T("StopLimit")));
}

void CBracketPropertiesDlg::initializeTargetOffsetTypes()
{
   int row = m_cmbTargetOffsetType.AddString(_T("Ticks"));
   m_cmbTargetOffsetType.SetItemData(row, int(eOffsetType::oftTicks));

   row = m_cmbTargetOffsetType.AddString(_T("Price"));
   m_cmbTargetOffsetType.SetItemData(row, int(eOffsetType::oftPrice));

   // Select first element
   m_cmbTargetOffsetType.SetCurSel(m_cmbTargetOffsetType.FindString(0, _T("Ticks")));
}

void CBracketPropertiesDlg::initializeStopLossOffsetType()
{
   int row = m_cmbStopLossOffsetType.AddString(_T("Ticks"));
   m_cmbStopLossOffsetType.SetItemData(row, int(eOffsetType::oftTicks));

   row = m_cmbStopLossOffsetType.AddString(_T("Price"));
   m_cmbStopLossOffsetType.SetItemData(row, int(eOffsetType::oftPrice));

   // Select first element
   m_cmbStopLossOffsetType.SetCurSel(m_cmbStopLossOffsetType.FindString(0, _T("Ticks")));
}

#pragma endregion


#pragma region message_handlers

void CBracketPropertiesDlg::OnBnClickedChkBrktIncludeStoploss()
{
   int isVisible = m_chkIncludeStopLoss.GetCheck() == BST_CHECKED ? SW_SHOW : SW_HIDE;

   m_cmbStopLossOffsetType.ShowWindow(isVisible);
   m_cmbStopLossOrderType.ShowWindow(isVisible);
   m_spnStopLossStopTickPrice.ShowWindow(isVisible);
   m_edStopLossStopTickPrice.ShowWindow(isVisible);

   if (static_cast<eOrderType>(m_cmbStopLossOrderType.GetItemData(m_cmbStopLossOrderType.GetCurSel())) == otStopLimit)
   {
      m_spnStopLossLimitTickPrice.ShowWindow(isVisible);
      m_edStopLossLimitTickPrice.ShowWindow(isVisible);
      m_lblStopLossOffsetType.ShowWindow(isVisible);
   }
}

void CBracketPropertiesDlg::OnBnClickedChkBrktIncludeTarget()
{
   int isVisible = m_chkIncludeTarget.GetCheck() == BST_CHECKED ? SW_SHOW : SW_HIDE;

   m_edTargetLimitTickPrice.ShowWindow(isVisible);
   m_spnTargetTickPrice.ShowWindow(isVisible);
   m_cmbTargetOffsetType.ShowWindow(isVisible);
}

void CBracketPropertiesDlg::OnCbnSelchangeCmbBrktStoplossOrdertype()
{
   eOrderType stopLossOrderType = static_cast<eOrderType>(m_cmbStopLossOrderType.GetItemData(m_cmbStopLossOrderType.GetCurSel()));
   int isVisible = stopLossOrderType == otStopLimit ? SW_SHOW : SW_HIDE;

   m_spnStopLossLimitTickPrice.ShowWindow(isVisible);
   m_edStopLossLimitTickPrice.ShowWindow(isVisible);
   m_lblStopLossOffsetType.ShowWindow(isVisible);
}

void CBracketPropertiesDlg::OnCbnSelchangeCmbBrktTargetOffsettype()
{
   if (static_cast<eOffsetType>(m_cmbStopLossOffsetType.GetItemData(m_cmbTargetOffsetType.GetCurSel())) == eOffsetType::oftTicks)
   {
      // Reset tick value in control
      m_edTargetLimitTickPrice.SetWindowText(_T("1"));
   }
   else
   {
      ATL::CComPtr<ICQGInstrument> instrument;
      HRESULT hr = m_spOrder->get_Instrument(&instrument);
      AssertCOMError(m_spOrder, hr);

      _bstr_t correctPrice;
      hr = instrument->ToDisplayPrice(getPrice(), correctPrice.GetAddress());
      AssertCOMError(instrument, hr);
      m_edTargetLimitTickPrice.SetWindowText(correctPrice);
   }
}

void CBracketPropertiesDlg::OnCbnSelchangeCmbBrktStoplossOffsettype()
{
   if (static_cast<eOffsetType>(m_cmbStopLossOffsetType.GetItemData(m_cmbStopLossOffsetType.GetCurSel())) == eOffsetType::oftTicks)
   {
      m_lblStopLossOffsetType.SetWindowText(_T("Ticks"));
      m_edStopLossStopTickPrice.SetWindowText(_T("1"));
      m_edStopLossLimitTickPrice.SetWindowText(_T("1"));
   }
   else
   {
      m_lblStopLossOffsetType.SetWindowText(_T("Price"));

      ATL::CComPtr<ICQGInstrument> instrument;
      HRESULT hr = m_spOrder->get_Instrument(&instrument);
      AssertCOMError(m_spOrder, hr);

      _bstr_t correctPrice;
      hr = instrument->ToDisplayPrice(getPrice(), correctPrice.GetAddress());
      AssertCOMError(instrument, hr);
      m_edStopLossStopTickPrice.SetWindowText(correctPrice);
      m_edStopLossLimitTickPrice.SetWindowText(correctPrice);
   }
}

void CBracketPropertiesDlg::OnBnClickedBtnBrktCancel()
{
   EndDialog(IDOK);
}

void CBracketPropertiesDlg::ChangeTick(CEdit& editCtrl, int delta)
{
   CString ticksStr;
   editCtrl.GetWindowText(ticksStr);
   int ticks = _ttoi(ticksStr);

   ticks -= delta;

   editCtrl.SetWindowText(_bstr_t(std::to_string(ticks).c_str()));
}

void CBracketPropertiesDlg::spinTickPrice(NMHDR* pNMHDR, LRESULT* pResult, CComboBox& cmbOffsetType, CEdit& edTickPrice)
{
   try
   {
      LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);

      ATL::CComPtr<ICQGInstrument> instrument;
      HRESULT hr = m_spOrder->get_Instrument(&instrument);
      AssertCOMError(m_spOrder, hr);

      if (cmbOffsetType.GetItemData(cmbOffsetType.GetCurSel()) == int(eOffsetType::oftPrice))
      {
         COrderPlacingDlg::ChangePrice(edTickPrice, instrument, pNMUpDown->iDelta);
      }
      else
      {
         ChangeTick(edTickPrice, pNMUpDown->iDelta);
      }
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
   }

   *pResult = 0;
}

void CBracketPropertiesDlg::OnDeltaposSpnBrktTargetTickprice(NMHDR *pNMHDR, LRESULT *pResult)
{
   spinTickPrice(pNMHDR, pResult, m_cmbTargetOffsetType, m_edTargetLimitTickPrice);
}

void CBracketPropertiesDlg::OnDeltaposSpnBrktStoplossStopTickprice(NMHDR *pNMHDR, LRESULT *pResult)
{
   spinTickPrice(pNMHDR, pResult, m_cmbStopLossOffsetType, m_edStopLossStopTickPrice);
}

void CBracketPropertiesDlg::OnDeltaposSpnBrktStoplossLimitTickprice(NMHDR *pNMHDR, LRESULT *pResult)
{
   spinTickPrice(pNMHDR, pResult, m_cmbStopLossOffsetType, m_edStopLossLimitTickPrice);
}

void CBracketPropertiesDlg::OnBnClickedBtnBrktPlace()
{
   eOrderType stopLossOrderType = static_cast<eOrderType>(m_cmbStopLossOrderType.GetItemData(m_cmbStopLossOrderType.GetCurSel()));

   ATL::CComPtr<ICQGBracketOrder> bracketOrder;
   HRESULT hr = m_spCEL->CreateBracketOrder(m_spOrder, stopLossOrderType, &bracketOrder);
   AssertCOMError(m_spCEL, hr);

   eSpeculationType speculationType = static_cast<eSpeculationType>(m_cmbSpeculationType.GetItemData(m_cmbSpeculationType.GetCurSel()));
   ATL::CComPtr<ICQGOrderProperties> mainOrderProperties;
   m_spOrder->get_Properties(&mainOrderProperties);
   COrderPlacingDlg::SetPropertyValue(mainOrderProperties, opSpeculationType, ATL::CComVariant(speculationType));

   bool includeTargetOrder = m_chkIncludeTarget.GetCheck() == BST_CHECKED;
   bool includeStopLossOrder = m_chkIncludeStopLoss.GetCheck() == BST_CHECKED;
   bracketOrder->put_IncludeTargetOrder(includeTargetOrder);
   bracketOrder->put_IncludeStopLossOrder(includeStopLossOrder);

   ATL::CComPtr<ICQGInstrument> instrument;
   m_spOrder->get_Instrument(&instrument);

   try
   {
      if (includeTargetOrder)
      {
         ATL::CComPtr<ICQGBracketOrderLeg> bracketLeg;
         bracketOrder->get_TargetLeg(&bracketLeg);

         if (m_cmbTargetOffsetType.GetItemData(m_cmbStopLossOffsetType.GetCurSel()) == int(eOffsetType::oftTicks))
         {
            CString tickStr;
            m_edTargetLimitTickPrice.GetWindowText(tickStr);
            bracketLeg->put_LimitOffset(_ttol(tickStr));
         }
         else
         {
            bracketLeg->put_LimitPrice(COrderPlacingDlg::GetPrice(m_edTargetLimitTickPrice, instrument));
         }
      }

      if (includeStopLossOrder)
      {
         ATL::CComPtr<ICQGBracketOrderLeg> bracketLeg;
         bracketOrder->get_StopLossLeg(&bracketLeg);

         if (m_cmbStopLossOffsetType.GetItemData(m_cmbStopLossOffsetType.GetCurSel()) == int(eOffsetType::oftTicks))
         {
            CString tickStr;
            m_edStopLossStopTickPrice.GetWindowText(tickStr);
            bracketLeg->put_StopOffset(_ttol(tickStr));
         }
         else
         {
            bracketLeg->put_StopPrice(COrderPlacingDlg::GetPrice(m_edStopLossStopTickPrice, instrument));
         }

         if (stopLossOrderType == otStopLimit)
         {
            if (m_cmbStopLossOffsetType.GetItemData(m_cmbStopLossOffsetType.GetCurSel()) == int(eOffsetType::oftTicks))
            {
               CString tickStr;
               m_edStopLossLimitTickPrice.GetWindowText(tickStr);
               bracketLeg->put_LimitOffset(_ttol(tickStr));
            }
            else
            {
               bracketLeg->put_LimitPrice(COrderPlacingDlg::GetPrice(m_edStopLossLimitTickPrice, instrument));
            }
         }
      }

      hr = bracketOrder->Place();
      AssertCOMError(bracketOrder, hr);
      OnOK();
   }
   catch (std::exception& e)
   {
      AfxMessageBox(ATL::CA2T(e.what()), MB_ICONSTOP);
      OnCancel();
   }
}

#pragma endregion
